### Persefone.jl - a model of agricultural landscapes and ecosystems in Europe.
###
### These are the tests for the crop growth models.
###

import Unitful

const TESTPARAM_ALMASS = joinpath(pkgdir(Persefone), "test", "test_parameters_almass.toml")
const TESTPARAM_SIMPLECROP = joinpath(pkgdir(Persefone), "test", "test_parameters_simplecrop.toml")
const TESTPARAM_AQUACROP = joinpath(pkgdir(Persefone), "test", "test_parameters_aquacrop.toml")

@testset for configfile in (TESTPARAM_ALMASS, TESTPARAM_SIMPLECROP, TESTPARAM_AQUACROP)
    @testset "Model initialisation" begin
        model = initialise(; configfile)
        @test model isa AgricultureModel
    end
    @testset "Time step" begin
        model = initialise(; configfile)
        stepsimulation!(model)
        @test model isa AgricultureModel
        # TODO: test stepagent!(), sow!(), harvest!()
    end
end

@testset "Submodule ALMaSS" begin
    ct = Ps.ALMaSS.CropType("olive tree", "no_crop_group", false, missing,
                            missing, missing, missing, missing, missing, missing, missing, 1.0)
    id = 0
    pixels = [(0, 0)]
    farmer = 0
    mature = false
    events = Ps.Management[]
    force_growth = false
    fp = FarmPlot(id, pixels, farmer, Ps.nosoildata, Ps.ALMaSS.CropState(croptype=ct, phase=Ps.ALMaSS.janfirst))
    @test fp isa FarmPlot
    @test fp.cropstate isa Ps.ALMaSS.CropState
    @test croptype(fp) isa Ps.ALMaSS.CropType
    @test cropname(fp) isa String
    @test cropheight(fp) isa Length{Float64}
    @test cropcover(fp) isa Float64
    @test cropyield(fp) isa Float64
end

@testset "Submodule SimpleCrop" begin
    ct = Ps.SimpleCrop.CropType("olive tree", "no_crop_group", AnnualDate(4, 1), AnnualDate(5, 1))
    id = 0
    pixels = [(0, 0)]
    farmer = 0
    fp = FarmPlot(id, pixels, farmer, Ps.nosoildata, Ps.SimpleCrop.CropState(ct, 0.0cm))
    @test fp isa FarmPlot
    @test fp.cropstate isa Ps.SimpleCrop.CropState
    @test croptype(fp) isa Ps.SimpleCrop.CropType
    @test cropname(fp) isa String
    @test cropheight(fp) isa Length{Float64}
    @test cropcover(fp) isa Float64
    @test cropyield(fp) isa Float64
end

@testset "Submodule AquaCrop" begin
    model = initialise(; configfile=TESTPARAM_AQUACROP)

    PsAC = Ps.AquaCropWrapper
    ct = PsAC.CropType("olive tree", "alfalfaGDD", "no_crop_group", AnnualDate(4, 1), AnnualDate(5, 1))
    id = 0
    pixels = [(0, 0)]
    farmer = 0
    soiltype = Ps.sand
    fp = FarmPlot(id, pixels, farmer, soiltype, PsAC.CropState(ct, soiltype, model))
    @test fp isa FarmPlot
    @test fp.cropstate isa PsAC.CropState
    @test croptype(fp) isa PsAC.CropType
    @test cropname(fp) isa String
    @test cropheight(fp) isa Length{Float64}
    @test cropcover(fp) isa Float64
    @test Unitful.dimension(cropyield(fp)) == Unitful.𝐌 * Unitful.𝐋^-2
end
